@@ -1,7 +1,7 @@
 from __future__ import print_function, division, absolute_import
 
 from .config import config
-from .core import connect, read, write, rpc
+from .core import connect, rpc
 from .deploy import LocalCluster
 from .diagnostics import progress
 from .client import (Client, Executor, CompatibleExecutor, wait, as_completed,
@@ -10,7 +10,7 @@ from .nanny import Nanny
 from .scheduler import Scheduler
 from .utils import sync
 from .worker import Worker
-from .worker_client import local_client
+from .worker_client import local_client, worker_client
 
 try:
     from .collections import futures_to_collection
