@@ -103,12 +103,11 @@ class Occupancy(DashboardComponent):
         with log_errors():
             o = self.scheduler.occupancy
             workers = list(self.scheduler.workers)
-            wi = self.scheduler.worker_info
-            try:
-                bokeh_addresses = [worker.split(':')[0] + ':' + str(wi[worker]['services']['bokeh'])
-                                   for worker in workers]
-            except Exception as e:
-                bokeh_addresses = [''] * len(workers)
+
+            bokeh_addresses = []
+            for worker in workers:
+                addr = self.scheduler.get_worker_service_addr(worker, 'bokeh')
+                bokeh_addresses.append('%s:%d' % addr if addr is not None else '')
 
             y = list(range(len(workers)))
             occupancy = [o[w] for w in workers]
@@ -239,6 +238,7 @@ def systemmonitor_doc(scheduler, doc):
     with log_errors():
         table = StateTable(scheduler)
         sysmon = SystemMonitor(scheduler, sizing_mode='scale_width')
+        doc.title = "Dask Scheduler Internal Monitor"
         doc.add_periodic_callback(table.update, 500)
         doc.add_periodic_callback(sysmon.update, 500)
 
@@ -253,6 +253,7 @@ def workers_doc(scheduler, doc):
         stealing_ts = StealingTimeSeries(scheduler, sizing_mode='scale_width')
         stealing_events = StealingEvents(scheduler, sizing_mode='scale_width')
         stealing_events.root.x_range = stealing_ts.root.x_range
+        doc.title = "Dask Workers Monitor"
         doc.add_periodic_callback(table.update, 500)
         doc.add_periodic_callback(occupancy.update, 500)
         doc.add_periodic_callback(stealing_ts.update, 500)
