@@ -4,6 +4,7 @@ import pytest
 pytest.importorskip('requests')
 
 from contextlib import contextmanager
+import itertools
 import os
 import requests
 import signal
@@ -11,19 +12,36 @@ import socket
 import sys
 from time import sleep
 
+from tornado import gen
 
 from distributed import Scheduler, Client
 from distributed.utils import get_ip, ignoring, tmpfile
-from distributed.utils_test import loop, popen
+from distributed.utils_test import (loop, popen,
+                                    assert_can_connect_from_everywhere_4,
+                                    assert_can_connect_from_everywhere_4_6,
+                                    assert_can_connect_locally_4,
+                                    )
 from distributed.metrics import time
 
 
 def test_defaults(loop):
     with popen(['dask-scheduler', '--no-bokeh']) as proc:
+
+        @gen.coroutine
+        def f():
+            # Default behaviour is to listen on all addresses
+            yield [
+                assert_can_connect_from_everywhere_4_6(8786, 2.0),  # main port
+                assert_can_connect_from_everywhere_4_6(9786, 2.0),  # HTTP port
+                ]
+
+        loop.run_sync(f)
+
         with Client('127.0.0.1:%d' % Scheduler.default_port, loop=loop) as c:
             response = requests.get('http://127.0.0.1:9786/info.json')
             assert response.ok
             assert response.json()['status'] == 'running'
+
     with pytest.raises(Exception):
         response = requests.get('http://127.0.0.1:9786/info.json')
     with pytest.raises(Exception):
@@ -31,7 +49,19 @@ def test_defaults(loop):
 
 
 def test_hostport(loop):
-    with popen(['dask-scheduler', '--no-bokeh', '--host', '127.0.0.1:8978']):
+    with popen(['dask-scheduler', '--no-bokeh', '--host', '127.0.0.1:8978',
+                '--http-port', '8979']):
+        @gen.coroutine
+        def f():
+            yield [
+                # The scheduler's main port can't be contacted from the outside
+                assert_can_connect_locally_4(8978, 2.0),
+                # ... but its HTTP port can
+                assert_can_connect_from_everywhere_4_6(8979, 2.0),
+                ]
+
+        loop.run_sync(f)
+
         with Client('127.0.0.1:8978', loop=loop) as c:
             assert len(c.ncores()) == 0
 
@@ -61,14 +91,17 @@ def test_bokeh(loop):
         start = time()
         while True:
             try:
+                # All addresses should respond
                 for name in names:
-                    response = requests.get('http://%s:8787/status/' % name)
+                    uri = 'http://%s:8787/status/' % name
+                    response = requests.get(uri)
                     assert response.ok
                 break
             except Exception as f:
-                print(f)
+                print('got error on %r: %s' % (uri, f))
                 sleep(0.1)
                 assert time() < start + 10
+
     with pytest.raises(Exception):
         requests.get('http://127.0.0.1:8787/status/')
 
@@ -96,6 +129,27 @@ def test_bokeh_non_standard_ports(loop):
         requests.get('http://localhost:4832/status/')
 
 
+def test_bokeh_internal_port(loop):
+    pytest.importorskip('bokeh')
+
+    with popen(['dask-scheduler', '--bokeh-internal-port', '4833']) as proc:
+        with Client('127.0.0.1:8786', loop=loop) as c:
+            d = c.scheduler_info()
+            assert d['services']['bokeh'] == 4833
+
+        start = time()
+        while True:
+            try:
+                response = requests.get('http://localhost:4833/workers/')
+                assert response.ok
+                break
+            except:
+                sleep(0.1)
+                assert time() < start + 20
+    with pytest.raises(Exception):
+        requests.get('http://localhost:4833/workers/')
+
+
 @pytest.mark.skipif(not sys.platform.startswith('linux'),
                     reason="Need 127.0.0.2 to mean localhost")
 def test_bokeh_whitelist(loop):
@@ -162,3 +216,25 @@ def test_pid_file(loop):
             with popen(['dask-worker', '127.0.0.1:8786', '--pid-file', w,
                         '--no-bokeh']) as worker:
                 check_pidfile(worker, w)
+
+
+def test_scheduler_port_zero(loop):
+    with tmpfile() as fn:
+        with popen(['dask-scheduler', '--no-bokeh', '--scheduler-file', fn,
+                    '--port', '0']) as sched:
+            with Client(scheduler_file=fn, loop=loop) as c:
+                assert c.scheduler.port
+                assert c.scheduler.port != 8786
+
+
+def test_bokeh_port_zero(loop):
+    with tmpfile() as fn:
+        with popen(['dask-scheduler',
+                    '--bokeh-port', '0',
+                    '--bokeh-internal-port', '0']) as proc:
+            count = 0
+            while count < 2:
+                line = proc.stderr.readline()
+                if b'bokeh' in line.lower() or b'web' in line.lower():
+                    count += 1
+                    assert b':0' not in line
