@@ -23,11 +23,12 @@ from ..utils import ignoring
 _deserialize = deserialize
 
 
-logger = logging.getLogger(__file__)
+logger = logging.getLogger(__name__)
 
 
 def dumps(msg):
     """ Transform Python message to bytestream suitable for communication """
+    original = msg
     try:
         data = {}
         # Only lists and dicts can contain serialized values
@@ -75,8 +76,6 @@ def dumps(msg):
             header['keys'].append(key)
             out_frames.extend(frames)
 
-        out_frames = [bytes(f) for f in out_frames]
-
         return [small_header, small_payload,
                 msgpack.dumps(header, use_bin_type=True)] + out_frames
     except Exception as e:
@@ -106,8 +105,11 @@ def loads(frames, deserialize=True):
             head = headers[key]
             lengths = head['lengths']
             count = head['count']
-            fs = frames[-count::][::-1]
-            del frames[-count:]
+            if count:
+                fs = frames[-count::][::-1]
+                del frames[-count:]
+            else:
+                fs = []
 
             if deserialize or key in bytestrings:
                 fs = decompress(head, fs)
