@@ -17,7 +17,7 @@ import tornado
 from tornado import gen
 from tornado.ioloop import TimeoutError
 
-from distributed.core import rpc, connect, read, write
+from distributed.core import rpc, connect
 from distributed.client import _wait
 from distributed.scheduler import Scheduler
 from distributed.metrics import time
@@ -27,7 +27,7 @@ from distributed.sizeof import sizeof
 from distributed.worker import Worker, error_message, logger, TOTAL_MEMORY
 from distributed.utils import ignoring, tmpfile
 from distributed.utils_test import (loop, inc, mul, gen_cluster, div,
-        slow, slowinc, throws, current_loop, gen_test, readone)
+        slow, slowinc, throws, gen_test, readone)
 
 
 
@@ -53,7 +53,7 @@ def test_identity():
     w = Worker('127.0.0.1', 8019)
     ident = w.identity(None)
     assert 'Worker' in ident['type']
-    assert ident['scheduler'] == ('127.0.0.1', 8019)
+    assert ident['scheduler'] == 'tcp://127.0.0.1:8019'
     assert isinstance(ident['ncores'], int)
     assert isinstance(ident['memory_limit'], Number)
 
@@ -194,8 +194,8 @@ def test_upload_file(c, s, a, b):
     assert not os.path.exists(os.path.join(b.local_dir, 'foobar.py'))
     assert a.local_dir != b.local_dir
 
-    aa = rpc(ip=a.ip, port=a.port)
-    bb = rpc(ip=b.ip, port=b.port)
+    aa = rpc(a.address)
+    bb = rpc(b.address)
     yield [aa.upload_file(filename='foobar.py', data=b'x = 123'),
            bb.upload_file(filename='foobar.py', data='x = 123')]
 
@@ -225,8 +225,8 @@ def test_upload_egg(c, s, a, b):
     assert not os.path.exists(os.path.join(b.local_dir, eggname))
     assert a.local_dir != b.local_dir
 
-    aa = rpc(ip=a.ip, port=a.port)
-    bb = rpc(ip=b.ip, port=b.port)
+    aa = rpc(a.address)
+    bb = rpc(b.address)
     with open(local_file, 'rb') as f:
         payload = f.read()
     yield [aa.upload_file(filename=eggname, data=payload),
@@ -252,7 +252,7 @@ def test_upload_egg(c, s, a, b):
 
 @gen_cluster()
 def test_broadcast(s, a, b):
-    with rpc(ip=s.ip, port=s.port) as cc:
+    with rpc(s.address) as cc:
         results = yield cc.broadcast(msg={'op': 'ping'})
         assert results == {a.address: b'pong', b.address: b'pong'}
 
@@ -260,22 +260,22 @@ def test_broadcast(s, a, b):
 @gen_test()
 def test_worker_with_port_zero():
     s = Scheduler()
-    s.listen(8007)
-    w = Worker(s.ip, s.port, ip='127.0.0.1')
+    s.start(8007)
+    w = Worker(s.ip, s.port)
     yield w._start()
     assert isinstance(w.port, int)
     assert w.port > 1024
 
 
 @slow
-def test_worker_waits_for_center_to_come_up(current_loop):
+def test_worker_waits_for_center_to_come_up(loop):
     @gen.coroutine
     def f():
-        w = Worker('127.0.0.1', 8007, ip='127.0.0.1')
+        w = Worker('127.0.0.1', 8007)
         yield w._start()
 
     try:
-        current_loop.run_sync(f, timeout=4)
+        loop.run_sync(f, timeout=4)
     except TimeoutError:
         pass
 
@@ -303,7 +303,7 @@ def test_error_message():
 def test_gather(s, a, b):
     b.data['x'] = 1
     b.data['y'] = 2
-    with rpc(ip=a.ip, port=a.port) as aa:
+    with rpc(a.address) as aa:
         resp = yield aa.gather(who_has={'x': [b.address], 'y': [b.address]})
         assert resp['status'] == 'OK'
 
@@ -315,14 +315,14 @@ def test_io_loop(loop):
     s = Scheduler(loop=loop)
     s.listen(0)
     assert s.io_loop is loop
-    w = Worker(s.ip, s.port, loop=loop)
+    w = Worker(s.address, loop=loop)
     assert w.io_loop is loop
 
 
 @gen_cluster(client=True, ncores=[])
 def test_spill_to_disk(e, s):
     np = pytest.importorskip('numpy')
-    w = Worker(s.ip, s.port, loop=s.loop, memory_limit=1000)
+    w = Worker(s.address, loop=s.loop, memory_limit=1000)
     yield w._start()
 
     x = e.submit(np.random.randint, 0, 255, size=500, dtype='u1', key='x')
