@@ -22,13 +22,13 @@ except ImportError:
 from tornado.gen import Return
 from tornado import gen
 from tornado.ioloop import IOLoop, PeriodicCallback
-from tornado.iostream import StreamClosedError
+from tornado.locks import Event
 
 from .batched import BatchedSend
+from .comm.core import get_address_host_port
 from .config import config
-from .utils_comm import pack_data, gather_from_workers
-from .compatibility import reload, unicode
-from .core import (read, write, connect, close, send_recv, error_message,
+from .compatibility import reload, unicode, invalidate_caches, cache_from_source
+from .core import (connect, send_recv, error_message, CommClosedError,
                    rpc, Server, pingpong, coerce_to_address, RPCClosed)
 from .metrics import time
 from .protocol.pickle import dumps, loads
@@ -36,6 +36,7 @@ from .sizeof import sizeof
 from .threadpoolexecutor import ThreadPoolExecutor
 from .utils import (funcname, get_ip, has_arg, _maybe_complex, log_errors,
                     All, ignoring, validate_key, mp_context)
+from .utils_comm import pack_data, gather_from_workers
 
 _ncores = mp_context.cpu_count()
 
@@ -63,12 +64,15 @@ READY = ('ready', 'constrained')
 
 class WorkerBase(Server):
 
-    def __init__(self, scheduler_ip, scheduler_port, ip=None, ncores=None,
+    def __init__(self, scheduler_ip, scheduler_port=None, ncores=None,
                  loop=None, local_dir=None, services=None, service_ports=None,
                  name=None, heartbeat_interval=5000, reconnect=True,
                  memory_limit='auto', executor=None, resources=None,
                  silence_logs=None, **kwargs):
-        self.ip = ip or get_ip()
+        if scheduler_port is None:
+            scheduler_addr = coerce_to_address(scheduler_ip)
+        else:
+            scheduler_addr = coerce_to_address((scheduler_ip, scheduler_port))
         self._port = 0
         self.ncores = ncores or _ncores
         self.local_dir = local_dir or tempfile.mkdtemp(prefix='worker-')
@@ -99,9 +103,10 @@ class WorkerBase(Server):
             self.data = dict()
         self.loop = loop or IOLoop.current()
         self.status = None
+        self._closed = Event()
         self.reconnect = reconnect
         self.executor = executor or ThreadPoolExecutor(self.ncores)
-        self.scheduler = rpc(ip=scheduler_ip, port=scheduler_port)
+        self.scheduler = rpc(scheduler_addr)
         self.name = name
         self.heartbeat_interval = heartbeat_interval
         self.heartbeat_active = False
@@ -117,15 +122,7 @@ class WorkerBase(Server):
 
         self.services = {}
         self.service_ports = service_ports or {}
-        for k, v in (services or {}).items():
-            if isinstance(k, tuple):
-                k, port = k
-            else:
-                port = 0
-
-            self.services[k] = v(self, io_loop=self.loop)
-            self.services[k].listen(port)
-            self.service_ports[k] = self.services[k].port
+        self.service_specs = services or {}
 
         handlers = {
           'gather': self.gather,
@@ -175,9 +172,11 @@ class WorkerBase(Server):
     def _register_with_scheduler(self):
         self.heartbeat_callback.stop()
         while True:
+            if self.status in ('closed', 'closing'):
+                raise gen.Return
             try:
                 resp = yield self.scheduler.register(
-                        ncores=self.ncores, address=(self.ip, self.port),
+                        ncores=self.ncores, address=self.address,
                         keys=list(self.data),
                         name=self.name,
                         nbytes=self.nbytes,
@@ -190,24 +189,48 @@ class WorkerBase(Server):
                 break
             except EnvironmentError:
                 logger.debug("Unable to register with scheduler.  Waiting")
-                yield gen.sleep(0.5)
+                yield gen.sleep(0.1)
         if resp != 'OK':
             raise ValueError(resp)
         self.heartbeat_callback.start()
 
-    @gen.coroutine
-    def _start(self, port=0):
-        self.listen(port)
-        self.name = self.name or self.address
-        for k, v in self.services.items():
-            v.listen(0)
-            self.service_ports[k] = v.port
+    def start_services(self, listen_ip=''):
+        for k, v in self.service_specs.items():
+            if isinstance(k, tuple):
+                k, port = k
+            else:
+                port = 0
 
-        logger.info('      Start worker at: %20s:%d', self.ip, self.port)
+            self.services[k] = v(self, io_loop=self.loop)
+            self.services[k].listen((listen_ip, port))
+            self.service_ports[k] = self.services[k].port
+
+    @gen.coroutine
+    def _start(self, addr_or_port=0):
+        assert self.status is None
+
+        # XXX Factor this out
+        if isinstance(addr_or_port, int):
+            # Default ip is the required one to reach the scheduler
+            self.ip = get_ip(
+                get_address_host_port(self.scheduler.address)[0]
+                )
+            self.listen((self.ip, addr_or_port))
+        else:
+            self.listen(addr_or_port)
+            self.ip = get_address_host_port(self.address)[0]
+
+        self.name = self.name or self.address
+        # Services listen on all addresses
+        # Note Nanny is not a "real" service, just some metadata
+        # passed in service_ports...
+        self.start_services()
+
+        logger.info('      Start worker at: %26s', self.address)
         for k, v in self.service_ports.items():
             logger.info('  %16s at: %20s:%d' % (k, self.ip, v))
-        logger.info('Waiting to connect to: %20s:%d',
-                    self.scheduler.ip, self.scheduler.port)
+        logger.info('Waiting to connect to: %26s',
+                    self.scheduler.address)
         logger.info('-' * 49)
         logger.info('              Threads: %26d', self.ncores)
         if self.memory_limit:
@@ -217,18 +240,17 @@ class WorkerBase(Server):
 
         yield self._register_with_scheduler()
 
-        logger.info('        Registered to: %20s:%d',
-                    self.scheduler.ip, self.scheduler.port)
+        logger.info('        Registered to: %32s', self.scheduler.address)
         logger.info('-' * 49)
         self.status = 'running'
 
     def start(self, port=0):
         self.loop.add_callback(self._start, port)
 
-    def identity(self, stream):
+    def identity(self, comm):
         return {'type': type(self).__name__,
                 'id': self.id,
-                'scheduler': (self.scheduler.ip, self.scheduler.port),
+                'scheduler': self.scheduler.address,
                 'ncores': self.ncores,
                 'memory_limit': self.memory_limit}
 
@@ -236,14 +258,14 @@ class WorkerBase(Server):
     def _close(self, report=True, timeout=10):
         if self.status in ('closed', 'closing'):
             return
-        logger.info("Stopping worker at %s:%d", self.ip, self.port)
+        logger.info("Stopping worker at %s", self.address)
         self.status = 'closing'
         self.stop()
         self.heartbeat_callback.stop()
         with ignoring(EnvironmentError):
             if report:
                 yield gen.with_timeout(timedelta(seconds=timeout),
-                        self.scheduler.unregister(address=(self.ip, self.port)),
+                        self.scheduler.unregister(address=self.address),
                         io_loop=self.loop)
         self.scheduler.close_rpc()
         self.executor.shutdown()
@@ -254,19 +276,17 @@ class WorkerBase(Server):
             v.stop()
         self.rpc.close()
         self.status = 'closed'
+        self._closed.set()
 
     @gen.coroutine
-    def terminate(self, stream, report=True):
+    def terminate(self, comm, report=True):
         yield self._close(report=report)
         raise Return('OK')
 
-    @property
-    def address(self):
-        return '%s:%d' % (self.ip, self.port)
-
-    @property
-    def address_tuple(self):
-        return (self.ip, self.port)
+    @gen.coroutine
+    def wait_until_closed(self):
+        yield self._closed.wait()
+        assert self.status == 'closed'
 
     def _deserialize(self, function=None, args=None, kwargs=None, task=None):
         """ Deserialize task inputs and regularize to func, args, kwargs """
@@ -308,14 +328,14 @@ class WorkerBase(Server):
         # logger.info("Finish job %d, %s", i, key)
         raise gen.Return(result)
 
-    def run(self, stream, function, args=(), kwargs={}):
-        return run(self, stream, function=function, args=args, kwargs=kwargs)
+    def run(self, comm, function, args=(), kwargs={}):
+        return run(self, comm, function=function, args=args, kwargs=kwargs)
 
-    def run_coroutine(self, stream, function, args=(), kwargs={}, wait=True):
-        return run(self, stream, function=function, args=args, kwargs=kwargs,
+    def run_coroutine(self, comm, function, args=(), kwargs={}, wait=True):
+        return run(self, comm, function=function, args=args, kwargs=kwargs,
                    is_coro=True, wait=wait)
 
-    def update_data(self, stream=None, data=None, report=True):
+    def update_data(self, comm=None, data=None, report=True):
         for key, value in data.items():
             if key in self.task_state:
                 self.transition(key, 'memory', value=value)
@@ -340,7 +360,7 @@ class WorkerBase(Server):
         return info
 
     @gen.coroutine
-    def delete_data(self, stream=None, keys=None, report=True):
+    def delete_data(self, comm=None, keys=None, report=True):
         if keys:
             for key in list(keys):
                 self.log.append((key, 'delete'))
@@ -358,7 +378,7 @@ class WorkerBase(Server):
         raise Return('OK')
 
     @gen.coroutine
-    def get_data(self, stream, keys=None, who=None):
+    def get_data(self, comm, keys=None, who=None):
         start = time()
 
         msg = {k: to_serialize(self.data[k]) for k in keys if k in self.data}
@@ -368,11 +388,10 @@ class WorkerBase(Server):
             self.digests['get-data-load-duration'].add(stop - start)
         start = time()
         try:
-            compressed = yield write(stream, msg)
-            yield close(stream)
+            compressed = yield comm.write(msg)
         except EnvironmentError:
             logger.exception('failed during get data', exc_info=True)
-            stream.close()
+            comm.abort()
             raise
         stop = time()
         if self.digests is not None:
@@ -396,7 +415,19 @@ class WorkerBase(Server):
 
         raise gen.Return('dont-reply')
 
-    def start_ipython(self, stream):
+    @gen.coroutine
+    def set_resources(self, **resources):
+        for r, quantity in resources.items():
+            if r in self.total_resources:
+                self.available_resources[r] += quantity - self.total_resources[r]
+            else:
+                self.available_resources[r] = quantity
+            self.total_resources[r] = quantity
+
+        yield self.scheduler.set_resources(resources=self.total_resources,
+                                           worker=self.address)
+
+    def start_ipython(self, comm):
         """Start an IPython kernel
 
         Returns Jupyter connection info dictionary.
@@ -410,7 +441,7 @@ class WorkerBase(Server):
             )
         return self._ipython_kernel.get_connection_info()
 
-    def upload_file(self, stream, filename=None, data=None, load=True):
+    def upload_file(self, comm, filename=None, data=None, load=True):
         out_filename = os.path.join(self.local_dir, filename)
         if isinstance(data, unicode):
             data = data.encode()
@@ -421,25 +452,37 @@ class WorkerBase(Server):
         if load:
             try:
                 name, ext = os.path.splitext(filename)
+                names_to_import = []
                 if ext in ('.py', '.pyc'):
-                    logger.info("Reload module %s from .py file", name)
-                    name = name.split('-')[0]
-                    reload(import_module(name))
-                if ext == '.egg':
-                    import pkg_resources
-                    sys.path.append(out_filename)
-                    pkgs = pkg_resources.find_distributions(out_filename)
-                    for pkg in pkgs:
-                        logger.info("Load module %s from egg", pkg.project_name)
-                        reload(import_module(pkg.project_name))
-                    if not pkgs:
-                        logger.warning("Found no packages in egg file")
+                    names_to_import.append(name)
+                    # Ensures that no pyc file will be reused
+                    cache_file = cache_from_source(out_filename)
+                    if os.path.exists(cache_file):
+                        os.remove(cache_file)
+                if ext in ('.egg', '.zip'):
+                    if out_filename not in sys.path:
+                        sys.path.insert(0, out_filename)
+                    if ext == '.egg':
+                        import pkg_resources
+                        pkgs = pkg_resources.find_distributions(out_filename)
+                        for pkg in pkgs:
+                            names_to_import.append(pkg.project_name)
+                    elif ext == '.zip':
+                        names_to_import.append(name)
+
+                if not names_to_import:
+                    logger.warning("Found nothing to import from %s", filename)
+                else:
+                    invalidate_caches()
+                    for name in names_to_import:
+                        logger.info("Reload module %s from %s file", name, ext)
+                        reload(import_module(name))
             except Exception as e:
                 logger.exception(e)
                 return {'status': 'error', 'exception': dumps(e)}
         return {'status': 'OK', 'nbytes': len(data)}
 
-    def host_health(self, stream=None):
+    def host_health(self, comm=None):
         """ Information about worker """
         d = {'time': time()}
         try:
@@ -476,20 +519,20 @@ class WorkerBase(Server):
             pass
         return d
 
-    def keys(self, stream=None):
+    def keys(self, comm=None):
         return list(self.data)
 
     @gen.coroutine
-    def gather(self, stream=None, who_has=None):
+    def gather(self, comm=None, who_has=None):
         who_has = {k: [coerce_to_address(addr) for addr in v]
                     for k, v in who_has.items()
                     if k not in self.data}
-        try:
-            result = yield gather_from_workers(who_has)
-        except KeyError as e:
+        result, missing_keys, missing_workers = yield gather_from_workers(
+                who_has, permissive=True)
+        if missing_keys:
             logger.warn("Could not find data", e)
             raise Return({'status': 'missing-data',
-                          'keys': e.args})
+                          'keys': missing_keys})
         else:
             self.update_data(data=result, report=False)
             raise Return({'status': 'OK'})
@@ -666,7 +709,7 @@ def weight(k, v):
 
 
 @gen.coroutine
-def run(server, stream, function, args=(), kwargs={}, is_coro=False, wait=True):
+def run(server, comm, function, args=(), kwargs={}, is_coro=False, wait=True):
     assert wait or is_coro, "Combination not supported"
     function = loads(function)
     if args:
@@ -744,7 +787,7 @@ class Worker(WorkerBase):
     * **total_connections**: ``int``
         The maximum number of concurrent connections we want to see
     * **total_comm_nbytes**: ``int``
-    * **batched_stream**: ``BatchedSend(IOstream)``
+    * **batched_stream**: ``BatchedSend``
         A batched stream along which we communicate to the scheduler
     * **log**: ``[(message)]``
         A structured and queryable log.  See ``Worker.story``
@@ -870,6 +913,7 @@ class Worker(WorkerBase):
         self.who_has = dict()
         self.has_what = defaultdict(set)
         self.pending_data_per_worker = defaultdict(deque)
+        self.extensions = {}
 
         self.data_needed = deque()  # TODO: replace with heap?
 
@@ -946,10 +990,10 @@ class Worker(WorkerBase):
     ################
 
     @gen.coroutine
-    def compute_stream(self, stream):
+    def compute_stream(self, comm):
         try:
             self.batched_stream = BatchedSend(interval=2, loop=self.loop)
-            self.batched_stream.start(stream)
+            self.batched_stream.start(comm)
 
             def on_closed():
                 if self.reconnect and self.status not in ('closed', 'closing'):
@@ -958,14 +1002,16 @@ class Worker(WorkerBase):
                 else:
                     self._close(report=False)
 
-            stream.set_close_callback(on_closed)
+            #stream.set_close_callback(on_closed)
 
             closed = False
 
             while not closed:
-                self.priority_counter += 1
                 try:
-                    msgs = yield read(stream)
+                    msgs = yield comm.read()
+                except CommClosedError:
+                    on_closed()
+                    break
                 except EnvironmentError as e:
                     break
 
@@ -981,8 +1027,6 @@ class Worker(WorkerBase):
                         break
                     elif op == 'compute-task':
                         priority = msg.pop('priority')
-                        priority = [self.priority_counter] + priority
-                        priority = tuple(-x for x in priority)
                         self.add_task(priority=priority, **msg)
                     elif op == 'release-task':
                         self.log.append((msg['key'], 'release-task'))
@@ -1296,10 +1340,10 @@ class Worker(WorkerBase):
             if self.batched_stream:
                 self.send_task_state_to_scheduler(key)
             else:
-                raise StreamClosedError()
+                raise CommClosedError
 
         except EnvironmentError:
-            logger.info("Stream closed")
+            logger.info("Comm closed")
         except Exception as e:
             logger.exception(e)
             if LOG_PDB:
@@ -1313,6 +1357,7 @@ class Worker(WorkerBase):
 
             self.executing.remove(key)
             self.long_running.add(key)
+            self.batched_stream.send({'op': 'long-running', 'key': key})
 
             self.ensure_computing()
         except Exception as e:
@@ -1474,16 +1519,15 @@ class Worker(WorkerBase):
     def gather_dep(self, worker, dep, deps, total_nbytes, cause=None):
         if self.status != 'running':
             return
-        stream = None
+        comm = None
         with log_errors():
-            ip, port = worker.split(':')
             response = {}
             try:
                 if self.validate:
                     self.validate_state()
 
                 start = time()
-                stream = yield connect(ip, int(port), timeout=3)
+                comm = yield connect(worker, timeout=3)
                 stop = time()
                 if self.digests is not None:
                     self.digests['gather-connect-duration'].add(stop - start)
@@ -1492,7 +1536,7 @@ class Worker(WorkerBase):
                 logger.debug("Request %d keys", len(deps))
 
                 start = time()
-                response = yield send_recv(stream, op='get_data', keys=list(deps),
+                response = yield send_recv(comm, op='get_data', keys=list(deps),
                                            close=True, who=self.address)
                 stop = time()
 
@@ -1532,8 +1576,8 @@ class Worker(WorkerBase):
                     import pdb; pdb.set_trace()
                 raise
             finally:
-                if stream:
-                    stream.close()
+                if comm:
+                    yield comm.close()
                 self.comm_nbytes -= total_nbytes
 
                 for d in self.in_flight_workers.pop(worker):
@@ -1618,11 +1662,14 @@ class Worker(WorkerBase):
                 import pdb; pdb.set_trace()
             raise
 
-    def release_key(self, key, cause=None):
+    def release_key(self, key, cause=None, reason=None):
         try:
             if key not in self.task_state:
                 return
             state = self.task_state.pop(key)
+            if reason == 'stolen' and state in ('executing', 'long-running', 'memory'):
+                self.task_state[key] = state
+                return
             if cause:
                 self.log.append((key, 'release-key', cause))
             else:
@@ -1664,7 +1711,7 @@ class Worker(WorkerBase):
                 self.batched_stream.send({'op': 'release',
                                           'key': key,
                                           'cause': cause})
-        except StreamClosedError:
+        except CommClosedError:
             pass
         except Exception as e:
             logger.exception(e)
@@ -1765,6 +1812,7 @@ class Worker(WorkerBase):
 
     @gen.coroutine
     def execute(self, key, report=False):
+        executor_error = None
         try:
             if key not in self.executing or key not in self.task_state:
                 return
@@ -1783,10 +1831,14 @@ class Worker(WorkerBase):
                 if self.digests is not None:
                     self.digests['disk-load-duration'].add(stop - start)
 
-            logger.debug("Execute key: %s", key)  # TODO: comment out?
-            result = yield self.executor_submit(key, apply_function, function,
-                                                args2, kwargs2,
-                                                self.execution_state, key)
+            logger.debug("Execute key: %s worker: %s", key, self.address)  # TODO: comment out?
+            try:
+                result = yield self.executor_submit(key, apply_function, function,
+                                                    args2, kwargs2,
+                                                    self.execution_state, key)
+            except RuntimeError as e:
+                executor_error = e
+                raise
 
             if self.task_state.get(key) not in ('executing', 'long-running'):
                 return
@@ -1827,13 +1879,14 @@ class Worker(WorkerBase):
 
             self.ensure_computing()
             self.ensure_communicating()
-        except RuntimeError as e:
-            logger.error("Thread Pool Executor error: %s", e)
         except Exception as e:
-            logger.exception(e)
-            if LOG_PDB:
-                import pdb; pdb.set_trace()
-            raise
+            if executor_error is e:
+                logger.error("Thread Pool Executor error: %s", e)
+            else:
+                logger.exception(e)
+                if LOG_PDB:
+                    import pdb; pdb.set_trace()
+                raise
         finally:
             if key in self.executing:
                 self.executing.remove(key)
